#include "ScriptWrapper.h"

QString ScriptWrapper::getlasterror()
{
	LPWSTR err = L"";
	DWORD iderr = GetLastError();
	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,NULL, iderr, 0, (LPWSTR)&err, 0, NULL);
	if (iderr == 0)
		err = L"";
	return QString::fromStdWString(err).trimmed();	
}

ScriptWrapper::ScriptWrapper()
{
	stopped = false;
}

ScriptWrapper::~ScriptWrapper()
{
	stopped = true;
}

void ScriptWrapper::clearConsole()
{
	emit clearMyConsole();
}

void ScriptWrapper::write(const QString &message_)
{
	emit WriteConsole(message_);
}

void ScriptWrapper::writeLn(const QString &message)
{
	emit WriteLnConsole(message);
}

void ScriptWrapper::showMessage(const QString &message)
{
	emit showMyMessage(message);
}

QDateTime ScriptWrapper::getFileDateTime(const QString & filename)
{
	QFileInfo fileinfo(filename);
	return fileinfo.lastModified();
}

bool ScriptWrapper::writeToFile(const QString & filename, const QString & content)
{
	QFile file(filename);

	if (!file.open(QFile::Append | QFile::Text))
		return false;

	QByteArray block;
	block.append(content+"\n");
	file.write(block);
	file.close();

	return true;
}

QString ScriptWrapper::fixRelativeUncPath(const QString &path)
{	
	QString path_ = QDir::convertSeparators(path);
	QString prefix = "\\\\?\\";

	if (path_.at(0) == '\\' && path_.at(1) == '\\' && path_.size() > 2)
	{
		prefix.append("UNC\\");
		path_.remove(0,2);
		return prefix+path_;
	}
	else
		return prefix+path_;			
}

bool ScriptWrapper::deleteFile_(const QString &filename)
{	
if (::SetNamedSecurityInfoW((wchar_t*)fixRelativeUncPath(filename).utf16(), SE_FILE_OBJECT, DACL_SECURITY_INFORMATION, NULL, NULL, NULL, NULL) != ERROR_SUCCESS)
	return false;

if (!::SetFileAttributesW((wchar_t*)fixRelativeUncPath(filename).utf16(),FILE_ATTRIBUTE_NORMAL))		
	return false;
		
if (!QFile().remove(filename))					
	return false; 

return true;
}

QString ScriptWrapper::copyFile(const QString& source, QString target) // 
{
	if (stopped)
		return " ";

	if ((source.isEmpty()) || (target.isEmpty()))
		return "    !";

	if (QDir().filePath(target).at(0)=='.') //  ,      
		target = QDir().currentPath()+"/"+target;

	if (source == target)
		return " =  !";
	
	QFileInfo fileinfo_s(source);
	QFile s(source);
	
	QFile t;
	if (target.lastIndexOf("\\") == target.size()-1 || target.lastIndexOf("/") == target.size()-1 )
		t.setFileName(target+fileinfo_s.fileName());
	else
		t.setFileName(target+QDir::separator()+fileinfo_s.fileName());

	if (!s.exists() && !fileinfo_s.isSymLink())
		return "   ! "+source;

	bool success = false;

	if (!QDir().exists(target)) 
	{
		if (!QDir().mkpath(target))
			return "  "+ target;
	}

	 if (fileinfo_s.isDir())
	 SetFileAttributesW((wchar_t*)fixRelativeUncPath(target).utf16(),::GetFileAttributesW((wchar_t*)fixRelativeUncPath(fileinfo_s.absolutePath()).utf16()));

	if (t.exists())		 
		if (!deleteFile_(t.fileName()))
			return "  " + t.fileName();


	HANDLE hSrc = ::CreateFileW(fixRelativeUncPath(source).utf16(), GENERIC_READ, FILE_SHARE_READ | FILE_SHARE_WRITE, 0, OPEN_EXISTING, 0, 0);
	if (hSrc == INVALID_HANDLE_VALUE)	       
		return "  "+source+" "+getlasterror(); 	  
		

	HANDLE hTgt = ::CreateFileW(fixRelativeUncPath(t.fileName()).utf16(), GENERIC_WRITE, 0, NULL, CREATE_ALWAYS, 0, 0);
	if (hTgt == INVALID_HANDLE_VALUE)
	{
		QString err = getlasterror();
		CloseHandle (hSrc);
		return "  "+t.fileName()+" "+err;		
	}

	DWORD read = 0;
	DWORD byteswritten = 0;
	memset(&buff,0,sizeof(buff));
	QString err;

	while (ReadFile (hSrc, buff, BUFFSIZE, &read, NULL) && read && !stopped)
	{
		success = WriteFile (hTgt, buff, read, &byteswritten, 0);
		err = getlasterror();

		success  &= (read == byteswritten);
		if (!success || !err.isEmpty())
		{
			LPCWSTR path = target.utf16();
			ULARGE_INTEGER FreeBytesAvailable;
			ULARGE_INTEGER TotalNumberOfBytes;
			ULARGE_INTEGER TotalNumberOfFreeBytes;
			GetDiskFreeSpaceEx( path, &FreeBytesAvailable, &TotalNumberOfBytes, &TotalNumberOfFreeBytes );
			qint64 bytesAvailable = FreeBytesAvailable.QuadPart;
			CloseHandle (hSrc);
			CloseHandle (hTgt);
			deleteFile_(t.fileName());				
			return " .   : "+t.fileName()+" "+err;						
		}
	}
	err = getlasterror();
	if (!err.isEmpty() || stopped)
	{
		CloseHandle (hSrc);
		CloseHandle (hTgt);
		deleteFile_(t.fileName());	
		if (stopped)
			return "     .   : "+t.fileName();
		else
			return "  -.   : "+t.fileName()+" "+err;		
	}

	FILETIME creationTime, accessTime, writeTime;
	GetFileTime(hSrc, &creationTime, &accessTime, &writeTime);		
	SetFileTime(hTgt, &creationTime, &accessTime, &writeTime);	
	CloseHandle (hSrc);
	CloseHandle (hTgt);

	if (!SetFileAttributesW((wchar_t*)fixRelativeUncPath(t.fileName()).utf16(),::GetFileAttributesW((wchar_t*)fixRelativeUncPath(source).utf16())))
		return "   "+t.fileName();
		
    return "";
}

QStringList ScriptWrapper::dir_entryList_files(const QString &dir)
{
	QStringList files_list;
	WIN32_FIND_DATA finfo = {0};
	HANDLE hFind = INVALID_HANDLE_VALUE;

	hFind = FindFirstFileW(QString(fixRelativeUncPath(dir)+"\\*").utf16(), &finfo);	
	if (hFind  != INVALID_HANDLE_VALUE)    
	{
		do 
		{
			if (lstrcmp(finfo.cFileName, L".") != 0 && lstrcmp(finfo.cFileName, L"..") != 0 )				
			{   			
				if (finfo.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)				
					continue;						   
				files_list.append(QString::fromStdWString(finfo.cFileName));
			}
		}
		while (!stopped && FindNextFileW(hFind, &finfo)); 
		FindClose(hFind); 		 
	}
	return files_list;
}

QStringList ScriptWrapper::dir_entryList_dirs(const QString &dir)
{
	QStringList dirs_list;
	WIN32_FIND_DATA finfo = {0}; 
	HANDLE hFind = INVALID_HANDLE_VALUE;

	hFind = FindFirstFileW(QString(fixRelativeUncPath(dir)+"\\*").utf16(), &finfo);
	if (hFind  != INVALID_HANDLE_VALUE)    
	{
		do 
		{
			if (lstrcmp(finfo.cFileName, L".") != 0 && lstrcmp(finfo.cFileName, L"..") != 0 )				
			{   			
				if (finfo.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)				
					dirs_list.append(QString::fromStdWString(finfo.cFileName));
			}
		}
		while (!stopped && FindNextFileW(hFind, &finfo)); 
		FindClose(hFind); 		 
	}
	return dirs_list;
}

void ScriptWrapper::copyDir_(const QString &Source, const QString &Destination, QStringList Filters, QString *err)
{		
	if (stopped)
	{
		*err = " ";
		return;
	}

	if (Source == Destination)	
	{
		*err = " =  !";	
		return;
	}

	QDir dir (Source);

	if (!dir.exists())
	{
		*err =  "    "+Source;
		return;
	}

	bool docopy = false;

	foreach (QString file, dir_entryList_files(Source))
	{				
		if (stopped)
		{
			*err = " ";
			return;
		}
	
		foreach(QString flt, Filters)
		{
			
			QRegExp rx_ext(flt);
			rx_ext.setPatternSyntax(QRegExp::Wildcard);
			if (rx_ext.exactMatch(file))
			{
				docopy = true;
				break;
			}

			/*if (PathMatchSpec(file.utf16(), flt.utf16()))
			{
				docopy = true;
				break;
			}*/
		}

		if (QFileInfo(dir.absolutePath()+QDir::separator()+file).isFile() || QFileInfo(dir.absolutePath()+QDir::separator()+file).isSymLink())
		{
			if (docopy)
			{
			QString temp_err = copyFile(dir.absolutePath()+QDir::separator()+file, Destination);
			if (temp_err != "")
				*err = *err + temp_err + "\n";
			docopy = false;
			}
		}
	}

		foreach (QString subDir, dir_entryList_dirs(Source))
	{		
		if (stopped)
		{
			*err = " ";
			return;
		}

		if (QFileInfo(Source+QDir::separator()+subDir).isDir())
			copyDir_(Source+QDir::separator()+subDir, Destination+QDir::separator()+subDir, Filters, err);		
	}
}

void ScriptWrapper::clearDir_(const QString &dirname, QString Filters, QString Age, QString *err)
{	
	if (stopped)
		return ;

	QDir dir (dirname);

	QStringList myfilters(Filters.split(";"));
	bool doclear = false;

	foreach (QString file, dir_entryList_files(dirname))
	{				
		if (stopped) 
			return ;

		foreach(QString flt, myfilters)
		{
			QRegExp rx_ext(flt);
			rx_ext.setPatternSyntax(QRegExp::Wildcard);
			if (rx_ext.exactMatch(file)) //    
			{
				 if (Age == "All")
				  {
				    doclear = true;
				    break;
				  }
				else
				{
					 QDate now = QDate::currentDate();
					 QFileInfo f(dir.absolutePath()+QDir::separator()+file);
					 int days = now.daysTo(f.lastModified().date());
					 if (days < 0 && qAbs(days) > Age.toInt())
					    doclear = true;
					 break;
				}
			}
		}

		if (QFileInfo(dir.absolutePath()+QDir::separator()+file).isFile() || QFileInfo(dir.absolutePath()+QDir::separator()+file).isSymLink())	//   4.7.4 ( - entryList        QDir::Hidden)	
			if (doclear)
			{
			if (!deleteFile_(dir.absolutePath()+QDir::separator()+file))				
				*err = *err + "  "+ dir.absolutePath()+QDir::separator()+file+"\n";			
			doclear = false;
			}
	}

	foreach (QString subDir, dir_entryList_dirs(dirname))
	{
		clearDir_(dirname+QDir::separator()+subDir, Filters, Age, err);
		DWORD attr = GetFileAttributesW((wchar_t*)fixRelativeUncPath(dirname+QDir::separator()+subDir).utf16());
		SetFileAttributesW((wchar_t*)fixRelativeUncPath(dirname+QDir::separator()+subDir).utf16(),FILE_ATTRIBUTE_NORMAL);
		if (!QDir().rmdir(dirname+QDir::separator()+subDir)) //      
			SetFileAttributesW((wchar_t*)fixRelativeUncPath(dirname+QDir::separator()+subDir).utf16(),attr);
	}
}

QString ScriptWrapper::copyDir(QString Source, QString Destination, QString Filters)
{
	QString err = "";
	
	if (Filters.isNull() || Filters == "")
		Filters = "*";

	QStringList myfilters(Filters.split(";"));

	while (Source.endsWith("\\") || Source.endsWith("/") )
		Source.remove(Source.size()-1,1);

	while (Destination.endsWith("\\") || Destination.endsWith("/") )
		Destination.remove(Destination.size()-1,1);
	
	copyDir_(Source, Destination, myfilters, &err);

	return err;
}

QString ScriptWrapper::deleteFile(const QString &fileName)
{
	if (deleteFile_(fileName))
		return "";
	else
		return "  "+ fileName;
}

QString ScriptWrapper::clearDir(QString dirname, QString Filters, QString Age)
{	
	QString err = "";

	if (Filters.isNull() || Filters == "")
		Filters = "*";

	bool ok;
	Age.toInt(&ok);
	if (!ok)
		Age = "All";

	while (dirname.endsWith("\\") || dirname.endsWith("/"))
		dirname.remove(dirname.size()-1,1);
		
	clearDir_(dirname, Filters, Age, &err);

	if (Filters == "*" && Age == "All")
	{
		if (!dirname.endsWith(":") && !dirname.endsWith("$") && dirname.size() >2)
		if (!QDir().rmdir(dirname))
			err = err +"  "+ dirname+"\n";
	}

	return err;
}

QString ScriptWrapper::executeFile(const QString &filename, const QString &app_arg, QString app_dir, const bool &wait)
{
	qint64 pid = -1;
	QFileInfo fi(filename);

	if (app_dir.isEmpty())
		app_dir = fi.absolutePath();

	if (!filename.isEmpty())
	{
		if (!QProcess::startDetached(filename,app_arg.split(" "),app_dir, &pid))
			return "  "+ filename;
		else
		{	
			if (wait)
			{
		    HANDLE hwnd = OpenProcess(MAXIMUM_ALLOWED, 0, pid);
		    if (!hwnd)
				return "   "+filename;
			WaitForSingleObject(hwnd, INFINITE); 
			CloseHandle(hwnd);
			}
			return "";
		}
	}
	else
		return "   ";
}

QString ScriptWrapper::runFile(const QString &filename, const QString &app_arg, QString app_dir, const bool &wait)
{
	int ret = -1;
	QFileInfo fi(filename);

	if (app_dir.isEmpty())
		app_dir = fi.absolutePath();

	if (filename.isEmpty())
		return "   ";

	if (wait)
	{
		QProcess process;
		QString OldPath = QDir::currentPath();
		QDir::setCurrent(app_dir);
		process.setWorkingDirectory(app_dir);
		ret = process.execute(filename,app_arg.split(" "));
		QDir::setCurrent(OldPath);
	}
	else
	{	
		HINSTANCE nResult;
		nResult = ShellExecute(NULL, L"open", filename.utf16(), app_arg.utf16(), app_dir.utf16(), SW_HIDE);
		if ((int) nResult <32)
			ret = -1;
    }

	if (ret < 0)
	return "   "+ filename;
	else
		return "";
}

void ScriptWrapper::quit()
{
	stopped = true;
}

void ScriptWrapper::setstart()
{
	stopped = false;
}

void ScriptWrapper::pause(const int &seconds)
{
	Sleep(seconds);
}

qint64 ScriptWrapper::GetDiskFreeSp(const QString &Disk)
{
	LPCWSTR path = Disk.utf16();
    ULARGE_INTEGER FreeBytesAvailable,TotalNumberOfBytes,TotalNumberOfFreeBytes;
	GetDiskFreeSpaceEx( path, &FreeBytesAvailable, &TotalNumberOfBytes, &TotalNumberOfFreeBytes );
	return FreeBytesAvailable.QuadPart / (1024*1024);
}

qint64 ScriptWrapper::GetDirSize(const QString &dir)
{
	if (stopped)
		return 0; 

	bool findlnk = false;

	if (QFileInfo(dir).isFile())				
		return QFileInfo(dir).size();

	qint64 size = 0; 
	WIN32_FIND_DATA finfo = {0};
	HANDLE hFind  = INVALID_HANDLE_VALUE;

	hFind = FindFirstFileW(QString(fixRelativeUncPath(dir)+"\\*").utf16(), &finfo);
	QString err = getlasterror();

	if (hFind  != INVALID_HANDLE_VALUE)    
	{
		do 
		{
			if (lstrcmp(finfo.cFileName, L".") != 0 && lstrcmp(finfo.cFileName, L"..") != 0 )				
			{   			
				if (finfo.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
				{
					if (finfo.dwFileAttributes & FILE_ATTRIBUTE_REPARSE_POINT) // 
						findlnk = true;
					else
						findlnk = false;

					if (!findlnk)
						size += GetDirSize(dir+"\\"+QString::fromStdWString(finfo.cFileName));

				}
				else			   
					size += ((DWORDLONG)finfo.nFileSizeHigh<<32) + finfo.nFileSizeLow;			  
			}
		}
		while (!stopped && FindNextFileW(hFind, &finfo)); 
		FindClose(hFind); 		 
	}
	return size;
}


QString ScriptWrapper::getLocation(const QString &location)
{	
	if (location == "Desktop")
	return QDesktopServices::storageLocation(QDesktopServices::DesktopLocation);
	else
		if (location == "Documents")		
	    return QDesktopServices::storageLocation(QDesktopServices::DocumentsLocation);
		else
			if (location == "Fonts")	
	        return QDesktopServices::storageLocation(QDesktopServices::FontsLocation);
			else
				if (location == "Applications")					
	            return QDesktopServices::storageLocation(QDesktopServices::ApplicationsLocation);
				else
					if (location == "Music")				
	                return QDesktopServices::storageLocation(QDesktopServices::MusicLocation);
					else
						if (location == "Movies")							
	                    return QDesktopServices::storageLocation(QDesktopServices::MoviesLocation);
						else
							if (location == "Pictures")
	                        return QDesktopServices::storageLocation(QDesktopServices::PicturesLocation);
							else
								if (location == "Temp")							
	                            return QDesktopServices::storageLocation(QDesktopServices::TempLocation);
									else
										if (location == "Home")
	                                    return QDesktopServices::storageLocation(QDesktopServices::HomeLocation);
										else
											if (location == "Data")
											return QDesktopServices::storageLocation(QDesktopServices::DataLocation);
											else
												if (location == "Cache")													
	                                            return QDesktopServices::storageLocation(QDesktopServices::CacheLocation);

	return "";
}
