#include "ScriptWrapper.h"

//ms_fnmatch.cpp
//http://www.gnu.org/software/libc/manual/html_node/Wildcard-Matching.html
//http://www.koders.com/cpp/fidEA06632025F95AC08C5B4A62188C68E1C99FF04D.aspx?s=include#L22
//http://bladebattles.com/kurok/SVN/psp/fnmatch.cpp

//http://publib.boulder.ibm.com/infocenter/zos/v1r12/index.jsp?topic=%2Fcom.ibm.zos.r12.bpxbd00%2Frfnmtc.htm

//http://code.google.com/p/shedskin/source/browse/trunk/shedskin/lib/fnmatch.cpp?r=1142


//http://doc.trolltech.com/qq/qq27-responsive-guis.html


QByteArray bb( 65536, ' ' );

ScriptWrapper::ScriptWrapper()
{
	stopped = false;
}

ScriptWrapper::~ScriptWrapper()
{
	stopped = true;
}

void ScriptWrapper::clearConsole()
{
	emit clearMyConsole();
}

void ScriptWrapper::writeToConsole(const QString &message_)
{
	emit WriteConsole(message_);
}

void ScriptWrapper::writeLnToConsole(const QString &message)
{
	emit WriteLnConsole(message);
}

void ScriptWrapper::showMessage(const QString &message)
{
	emit showMyMessage(message);
}

QDateTime ScriptWrapper::getFileDateTime(const QString & filename)
{
	QFileInfo fileinfo(filename);
	return fileinfo.lastModified();
}

bool ScriptWrapper::writeToFile(const QString & filename, const QString & content)
{
	QFile file(filename);

	if (!file.open(QFile::WriteOnly | QFile::Text))
		return false;

	QByteArray block;
	block.append(content);
	file.write(block);
	file.close();

	return true;
}

bool ScriptWrapper::copyData(QIODevice& src, QIODevice& dest)
{
	bool success = false;

	if ((src.openMode() & QIODevice::ReadOnly) == 0)
		return false;

	if ((dest.openMode() & QIODevice::WriteOnly) == 0)
		return false;

	bb.fill(' ');

	if (bb.size() > 0) //   
	{
		qint64 byteswritten;
		qint64 bytesread = src.read( bb.data(), bb.size() );

		if (bytesread == 0 && src.size() == 0)
		{
			byteswritten = dest.write( bb.data(), bytesread );
			return success = (bytesread == byteswritten);				
		}

		success = (bytesread > 0);

		while (bytesread > 0 && success)
		{					     		
			if (stopped)						 
				return false;

			byteswritten = dest.write( bb.data(), bytesread );
			success  &= (bytesread == byteswritten);
			bytesread = src.read(bb.data(), bb.size());
		}
	}
	return success;
}


bool ScriptWrapper::deleteFile(const QString &filename)
{	
if (::SetNamedSecurityInfo((wchar_t*)QString(filename).utf16(), SE_FILE_OBJECT, DACL_SECURITY_INFORMATION, NULL, NULL, NULL, NULL) != ERROR_SUCCESS)
	return false;

if (!::SetFileAttributes((wchar_t*)QString(filename).utf16(),FILE_ATTRIBUTE_NORMAL))		
	return false;
		
if (!QFile().remove(filename))					
	return false; 

return true;
}


QString ScriptWrapper::copyFile(const QString& source, const QString& target) // 
{

	if (stopped)
		return " ";
	
	bool success = false;

	if ((source.isEmpty()) || (target.isEmpty()))
		return "    !";

	if (source == target)
		return " =  !";
	
	QFileInfo fileinfo_s(source);
	QFile s(source);
	
	QFile t;
	if (target.lastIndexOf("\\") == target.size()-1 || target.lastIndexOf("/") == target.size()-1 )
		t.setFileName(target+fileinfo_s.fileName());
	else
		t.setFileName(target+QDir::separator()+fileinfo_s.fileName());

	if (!s.exists() && !fileinfo_s.isSymLink())
		return "   ! "+source;

	if (!QDir().exists(target)) 
	{
		if (!QDir().mkpath(target))
			return "  "+ target;
	}

	::SetFileAttributes((wchar_t*)QString(target).utf16(),::GetFileAttributes((wchar_t*)QString(fileinfo_s.absolutePath()).utf16()));	

	if (t.exists())		 
		if (!deleteFile(t.fileName()))
			return "  " + t.fileName();

	if (s.open(QIODevice::ReadOnly))
	{
		if (t.open(QIODevice::WriteOnly))
		{
			success  = copyData(s, t);
			success &= (s.error() == QFile::NoError && t.error() == QFile::NoError);
			t.close();					
		}
		s.close();

		if (!success)
		{
			t.remove();
			return "  .   : " + t.fileName();
		}

			FILETIME creationTime, accessTime, writeTime;						
			HANDLE hSrc = ::CreateFile(QDir::convertSeparators(source).utf16(), FILE_READ_ATTRIBUTES, FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE, NULL, OPEN_EXISTING, 0, 0);
			HANDLE hTgt = ::CreateFile(QDir::convertSeparators(t.fileName()).utf16(), GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, 0);

			GetFileTime(hSrc, &creationTime, &accessTime, &writeTime);
			SetFileTime(hTgt, &creationTime, &accessTime, &writeTime);
						
			CloseHandle(hSrc);
			CloseHandle(hTgt);

			SetFileAttributes((wchar_t*)QString(t.fileName()).utf16(),::GetFileAttributes((wchar_t*)(source.utf16())));				
	}
	else
	{
		return "    " + source;
	}

	if (!success)
		return " ";
	else
		return "";
}


void ScriptWrapper::copyDir_(const QString &Source, const QString &Destination, QStringList Filters, QString *err)
{		
	if (stopped)
	{
		*err = " ";
		return;
	}

	if (Source == Destination)	
	{
		*err = " =  !";	
		return;
	}

	QDir dir (Source);

	if (!dir.exists())
	{
		*err =  "    "+Source;
		return;
	}

	bool docopy = false;

	foreach (QString file, dir.entryList(QDir::Files | QDir::Hidden | QDir::System))
	{				
		if (stopped)
		{
			*err = " ";
			return;
		}
	
		foreach(QString flt, Filters)
		{
			
			QRegExp rx_ext(flt);
			rx_ext.setPatternSyntax(QRegExp::Wildcard);
			if (rx_ext.exactMatch(file))
			{
				docopy = true;
				break;
			}

			/*if (PathMatchSpec(file.utf16(), flt.utf16()))
			{
				docopy = true;
				break;
			}*/
		}

		if (QFileInfo(dir.absolutePath()+QDir::separator()+file).isFile())
		{
			if (docopy)
			{
			QString temp_err = copyFile(dir.absolutePath()+QDir::separator()+file, Destination);
			if (temp_err != "")
				*err = *err + temp_err + "\n";
			docopy = false;
			}
		}
	}

	foreach (QString subDir, dir.entryList(QDir::NoDotAndDotDot|QDir::Dirs| QDir::Hidden))
	{		
		if (stopped)
		{
			*err = " ";
			return;
		}

		if (QFileInfo(Source+QDir::separator()+subDir).isDir())
			copyDir_(Source+QDir::separator()+subDir, Destination+QDir::separator()+subDir, Filters, err);		
	}
}


void ScriptWrapper::clearDir_(const QString &dirname, QString Filters, QString *err)
{	
	if (stopped)
		return ;

	QDir dir (dirname);

	QStringList myfilters(Filters.split(";"));
	bool doclear = false;

	foreach (QString file, dir.entryList(QDir::Files | QDir::Hidden | QDir::System))  //   
	{				

		if (stopped) 
			return ;

		foreach(QString flt, myfilters)
		{
			QRegExp rx_ext(flt);
			rx_ext.setPatternSyntax(QRegExp::Wildcard);
			if (rx_ext.exactMatch(file)) //    
			{
				doclear = true;
				break;
			}

			/*if (PathMatchSpec(file.utf16(), flt.utf16()))
			{
				doclear = true;
				break;
			}*/
		}

		if (QFileInfo(dir.absolutePath()+QDir::separator()+file).isFile() || QFileInfo(dir.absolutePath()+QDir::separator()+file).isSymLink())	//   4.7.4 ( - entryList        QDir::Hidden)	
			if (doclear)
			{
			if (!deleteFile(dir.absolutePath()+QDir::separator()+file))				
				*err = *err + "  "+ dir.absolutePath()+QDir::separator()+file+"\n";			
			doclear = false;
			}
	}

	foreach (QString subDir, dir.entryList(QDir::NoDotAndDotDot|QDir::Dirs| QDir::Hidden))	//  
	{
		clearDir_(dirname+QDir::separator()+subDir, Filters, err);

		if (Filters == "*")
		{
		if (!QDir().rmdir(dirname+QDir::separator()+subDir))
			*err = *err +"  "+ dirname+QDir::separator()+subDir+"\n";
		}
	}
}

QString ScriptWrapper::copyDir(const QString &Source, const QString &Destination, QString Filters)
{
	QString err = "";
	
	if (Filters.isNull() || Filters == "")
		Filters = "*";

	QStringList myfilters(Filters.split(";"));
	
	copyDir_(Source, Destination, myfilters, &err);

	return err;
}

QString ScriptWrapper::clearDir(const QString &dirname, QString Filters)
{	
	QString err = "";

	if (Filters.isNull() || Filters == "")
		Filters = "*";
		
	clearDir_(dirname, Filters ,&err);

	if (Filters == "*")
	{
		if (!QDir().rmdir(dirname))
			err = err +"  "+ dirname+"\n";
	}

	return err;
}

QString ScriptWrapper::executeFile(const QString &filename, const QString &app_arg, QString app_dir)
{
	QFileInfo fi(filename);
	if (app_dir.isEmpty())
		app_dir = fi.absolutePath();

	if (!filename.isEmpty())
	{
		if (!QProcess::startDetached(filename,app_arg.split(" "),app_dir))
			return "  "+ filename;
		else	
			return "";
	}
	else
		return "   ";
}

void ScriptWrapper::quit()
{
	stopped = true;
}

void ScriptWrapper::setstart()
{
	stopped = false;
}

void ScriptWrapper::pause(const int &seconds)
{
	Sleep(seconds);
}


QString ScriptWrapper::getLocation(const QString &location)
{	
	if (location == "Desktop")
	return QDesktopServices::storageLocation(QDesktopServices::DesktopLocation);
	else
		if (location == "Documents")		
	    return QDesktopServices::storageLocation(QDesktopServices::DocumentsLocation);
		else
			if (location == "Fonts")	
	        return QDesktopServices::storageLocation(QDesktopServices::FontsLocation);
			else
				if (location == "Applications")					
	            return QDesktopServices::storageLocation(QDesktopServices::ApplicationsLocation);
				else
					if (location == "Music")				
	                return QDesktopServices::storageLocation(QDesktopServices::MusicLocation);
					else
						if (location == "Movies")							
	                    return QDesktopServices::storageLocation(QDesktopServices::MoviesLocation);
						else
							if (location == "Pictures")
	                        return QDesktopServices::storageLocation(QDesktopServices::PicturesLocation);
							else
								if (location == "Temp")							
	                            return QDesktopServices::storageLocation(QDesktopServices::TempLocation);
									else
										if (location == "Home")
	                                    return QDesktopServices::storageLocation(QDesktopServices::HomeLocation);
										else
											if (location == "Data")
											return QDesktopServices::storageLocation(QDesktopServices::DataLocation);
											else
												if (location == "Cache")													
	                                            return QDesktopServices::storageLocation(QDesktopServices::CacheLocation);

	return "";
}
