#include "add_task.h"
#include "new_scheduller.h"

const int delta = 45;

Highlighter::Highlighter(QTextDocument *parent)
	: QSyntaxHighlighter(parent)
{

}

void Highlighter::highlightBlock(const QString &text)
{
	int index = 0;
	QString singleLineComment = "//[^\n]*";

	QStringList keywordPatterns;

	keywordPatterns  << "NaN" <<"abstract" <<"boolean" <<"break" <<"byte" 
					 <<"case" <<"catch" <<"char" <<"class" <<"const" 
					 <<"constructor" <<"continue" <<"debugger" <<"default" <<"delete" 
					 <<"do" <<"double" <<"else" <<"enum" <<"export" 
					 <<"extends" <<"false" <<"final" <<"finally" <<"float" <<"for"
					 <<"function" <<"goto" <<"if" <<"implements" <<"import" 
					 <<"in" <<"instanceof" <<"int" <<"interface" <<"long"
					 <<"native" <<"new" <<"package" <<"private" <<"protected" 
					 <<"public" <<"return" <<"short" <<"static" <<"super" 
					 <<"switch" <<"synchronized" <<"this" <<"throw" <<"throws" 
					 <<"transient" <<"true" <<"try" <<"typeof" <<"undefined" 
					 <<"var" <<"void" <<"volatile" <<"while" <<"with"<<"Infinity"
					 <<"engine"<<"writeLn"<<"write"<<"clearConsole"
					 <<"showMessage"<<"getFileDateTime"<<"writeToFile"<<"copyFile"
					 <<"copyDir"<<"clearDir"<<"executeFile" <<"getLocation"<<"pause" 
					 <<"runFile"<<"deleteFile" <<"GetDiskFreeSp" <<"GetDirSize";


	foreach (const QString &pattern, keywordPatterns) //   
	{
		index = text.indexOf(pattern);
		while (index >= 0)
		{
			int length = pattern.size();
			if (text.size() == length)
				setFormat(index, length, Qt::darkYellow);
			else
				if (text.size() > index+length)
				{
					if (text.at(index+length) == '(' || text.at(index+length) == '.' || text.at(index+length) == ' ' || text.at(index+length) == '[' || text.at(index+length) == ',' 
						|| text.at(index+length) == '{')
					{
						if (index == 0) 
							setFormat(index, length, Qt::darkYellow);
						else
							if (index > 0 )
								if (text.at(index-1) == ')' || text.at(index-1) == '(' || text.at(index-1) == '[' || text.at(index-1) == '{' 
									|| text.at(index-1) == '.' || text.at(index-1) == ' ' || text.at(index-1) == ']' || text.at(index-1) == ',' || text.at(index-1) == '}')
									setFormat(index, length, Qt::darkYellow);
					}
				}

			index = text.indexOf(pattern, index + length);
		}	
	}

	
	index = text.indexOf('"'); // 
	int oldindex = 0;
	int count = 0;

	while (index >= 0)
	{
		count ++;
		if (count != 2)	
			oldindex = index;
		else
		{
			setFormat(oldindex + 1, index-1 - oldindex, Qt::darkGreen);
			count = 0;
		}
		index = text.indexOf('"', index + 1);
	}	

	QRegExp exp_singleLineComment(singleLineComment);	//  
	index = 0;
	while  ((index = exp_singleLineComment.indexIn(text, index)) != -1)
	{
		QTextCharFormat myClassFormat;
		myClassFormat.setForeground(Qt::darkGreen);

		setFormat (index, exp_singleLineComment.matchedLength(), myClassFormat);
		index += exp_singleLineComment.matchedLength();	
	}


	QTextCharFormat multiLineCommentFormat;				//  
	multiLineCommentFormat.setForeground(Qt::darkGreen);

	QRegExp startExpression("/\\*");
	QRegExp endExpression("\\*/");

	setCurrentBlockState(0);

	int startIndex = 0;
	if (previousBlockState() != 1)
		startIndex = text.indexOf(startExpression);

	while (startIndex >= 0) 
	{
		int endIndex = text.indexOf(endExpression, startIndex);
		int commentLength;
		if (endIndex == -1) 
		{
			setCurrentBlockState(1);
			commentLength = text.length() - startIndex;
		} else {
			commentLength = endIndex - startIndex
				+ endExpression.matchedLength();
		}
		setFormat(startIndex, commentLength, multiLineCommentFormat);
		startIndex = text.indexOf(startExpression,
			startIndex + commentLength);
	}
}

Editor::Editor(QWidget *parent/* =0 */)
{
	setViewportMargins(delta, 0, 0, 0); 
	connect(verticalScrollBar(), SIGNAL(valueChanged(int)), this, SLOT(update()));
	connect(this, SIGNAL(textChanged()), this, SLOT(update()));
	connect(this, SIGNAL(cursorPositionChanged()),	this, SLOT(curposition())); 	
}

bool Editor::event(QEvent *event)  
{
	if (event->type() == QEvent::Paint) 
	{
		int index = 0;
		QPainter painter(this); 
		painter.fillRect(0, 0, delta, height(), QColor(255, 250, 170));
		
		int vscrollValue = verticalScrollBar()->value();
		qreal pageBottom = vscrollValue + viewport()->height();
		int lineNumber(1);
		int currentLine = textCursor().blockNumber()+1; 
		const QFontMetrics fm=fontMetrics();
		const int fontAscent = fontMetrics().ascent() + 1; 

		for (QTextBlock block = document()->begin(); block.isValid(); block = block.next(), lineNumber++) 
		{
			QTextLayout *layout = block.layout();
			const QRectF boundingRect = layout->boundingRect();
			QPointF position = layout->position();

			QTextCharFormat fmt;
			fmt.setBackground(QColor(0, 250, 205));			

			if ( position.y() + boundingRect.height() < vscrollValue )
				continue;
			if ( position.y() > pageBottom ) 
				break;


			const QString str = QString::number(lineNumber);
			painter.drawText(delta - fm.width(str) - 3, qRound(position.y()) - vscrollValue + fontAscent, str); 
		} 

	}
	 return QTextEdit::event(event); 	
}

void Editor::paintEvent(QPaintEvent *e)
{
	QPainter p(viewport());
	QRect r = cursorRect();
	r.setX(0);
	r.setWidth(viewport()->width());
	p.fillRect(r, QBrush(QColor(255, 250, 180)));
	QTextEdit::paintEvent( e );
}

Editor::~Editor()
{

}

void Editor::curposition()
{ 

	viewport()->update(); 
	//	repaint();	
} 

add_my_task::add_my_task(QWidget *parent): QDialog(parent)
{
	setupUi(this);
	itemindex=-1;
	edit = false;
	//setWindowModality(Qt::ApplicationModal);
	popupcal=new QCalendarWidget();
	popupcal->setVerticalHeaderFormat(QCalendarWidget::NoVerticalHeader);
	popupcal->setHorizontalHeaderFormat(QCalendarWidget::ShortDayNames);
	popupcal->setFirstDayOfWeek(Qt::Monday);
	ex_Date_Copy->setCalendarWidget(popupcal);

	connect(include_filter, SIGNAL(clicked()), this, SLOT(set_item_filter_state()));
	connect(exclude_filter, SIGNAL(clicked()), this, SLOT(set_item_filter_state()));
	connect(listWidget, SIGNAL(currentRowChanged (int)), this, SLOT(set_CheckBox_state(int)));
	connect(enable_concurrent_execution, SIGNAL (toggled(bool)), this, SLOT(set_state(bool)));
	connect(disable_concurrent_execution, SIGNAL (toggled(bool)), this, SLOT(set_state(bool)));

	QRegExp rx("[0-9]+");
	v = new QRegExpValidator(rx, this);
	ex_Size_Copy->setValidator(v);
	//ex_Date_Copy->setDisplayFormat("yyyy");

	engine = new QScriptEngine(this);
	my_ScriptWrapper = new ScriptWrapper;

	QScriptValue ret = engine->importExtension("qt.core"); 
	if (ret.isError())
		QMessageBox::information(0, " !", "    ");

	QScriptValue my_Wrapper = engine->newQObject(my_ScriptWrapper);
	engine->globalObject().setProperty("engine", my_Wrapper);
	if (my_Wrapper.isError())
		QMessageBox::information(0, " !", "  ");


	scriptEdior = new Editor(groupBox_6);
	gridLayout_11->addWidget(scriptEdior, 0, 1, 1, 3);

	QPushButton *LoadButton = new QPushButton(groupBox_6);
	LoadButton->setText("  ");	
	gridLayout_11->addWidget(LoadButton, 1, 1, 1, 1);
	connect(LoadButton, SIGNAL(clicked()), this, SLOT(load_script()));

	QPushButton *DebugButton = new QPushButton(groupBox_6);
	DebugButton->setText("  ");	
	gridLayout_11->addWidget(DebugButton, 1, 2, 1, 1);
	connect(DebugButton, SIGNAL(clicked()), this, SLOT(debug_script()));

	QPushButton *SaveButton = new QPushButton(groupBox_6);
	SaveButton->setText("  ");	
	gridLayout_11->addWidget(SaveButton, 1, 3, 1, 1);
	connect(SaveButton, SIGNAL(clicked()), this, SLOT(save_script()));

	Highlighter *highlighter;
	highlighter = new Highlighter(scriptEdior->document());

}

add_my_task::~add_my_task()
{
	if (popupcal)
	{
		delete popupcal;
		popupcal=0;
	}

	if (v)
	{
		delete v;
		v = 0;
	}

	if (engine)
	{
		delete engine;
		engine = 0;
	}

	if (scriptEdior)
	{
		delete scriptEdior;
		scriptEdior = 0;
	}

	if (my_ScriptWrapper)
	{
		delete my_ScriptWrapper;
		my_ScriptWrapper = 0;
	}
}

void add_my_task::set_state(bool state)
{
	if (disable_concurrent_execution->isChecked())
	{
		label_4->setEnabled(true);
		comboBox_2->setEnabled(true);
		groupBox_12->setEnabled(true);
		stackedWidget_3->setEnabled(true);

		int old_index = comboBox_2->currentIndex();
		comboBox_2->setCurrentIndex(-1); //  on_comboBox_2_currentIndexChanged     
		comboBox_2->setCurrentIndex(old_index);
	}
	else
	{
		label_4->setEnabled(false);
		comboBox_2->setEnabled(false);
		groupBox_12->setEnabled(false);		
		stackedWidget_3->setEnabled(false);
	}	
}

void add_my_task::form_clear()
{
	groupBox_6->setTitle("");
	task_name->clear();
	task_name_2->clear();
	scriptEdior->clear();
	list_files_and_dirs->clear();
	dest_Edit->clear();
	create_add_dir->setChecked(0);
	additional_dir->clear();
	listWidget->clear();
	include_filter->setChecked(0);
	exclude_filter->setChecked(0);
	do_before->setChecked(0);
	execute_before->clear();
	do_after->setChecked(0);
	execute_after->clear();
	exDate->setChecked(0);
	execute_before_cmd->clear();
	execute_before_dir->clear();
	execute_after_cmd->clear();
	execute_after_dir->clear();
	conditionCombo->setCurrentIndex(0);
	conditionCombo_2->setCurrentIndex(0);
	sizeCombo->setCurrentIndex(0);
	ex_Date_Copy->setDate(QDate::currentDate());
	ex_Size_Copy->clear();
	exSize->setChecked(0);
	notRewrite->setChecked(0);
	reWrite->setChecked(0);
	clearDest->setChecked(0);
	cmp_md5->setChecked(0);
}

void add_my_task::on_nextButton_clicked()
{
	int index = 0;

	if (radioButton->isChecked())
		index = 1;
	else 
		if (radioButton_2->isChecked())
			index = 2;
			
	stackedWidget->setCurrentIndex(index);
}

void add_my_task::on_cancelButton_1_clicked()
{
	close();
}

void add_my_task::on_cancelButton_2_clicked()
{
	close();
}

void add_my_task::on_cancelButton_3_clicked()
{
	close();
}

void add_my_task::on_cancelButton_4_clicked()
{
	close();
}

void add_my_task::on_cancelButton_5_clicked()
{
	close();
}


void add_my_task::on_backButton_1_clicked()
{
	if (!edit)
	form_clear();
	stackedWidget->setCurrentIndex(0);
}

void add_my_task::on_backButton_2_clicked()
{
	stackedWidget->setCurrentIndex(0);
}

void add_my_task::add_files()
{
	QStringList inputfiles=QFileDialog::getOpenFileNames(this,tr("  ?"),lastOpenDir,"All (*.*)",0,QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
	if (!inputfiles.isEmpty())
	{
	list_files_and_dirs->addItems(inputfiles);
	lastOpenDir = QFileInfo(inputfiles.at(0)).absoluteDir().path();
	}
}

void add_my_task::add_dirs()
{	
	QString directory = QFileDialog::getExistingDirectory(this,tr("  ?"),lastOpenDir,QFileDialog::ShowDirsOnly);
	if (!directory.isEmpty())
	{
		list_files_and_dirs->addItem(directory);
		QDir updir(directory);
		updir.cdUp();
		lastOpenDir = updir.path();		
	}
}

void add_my_task::load_script()
{	
	QString scriptpath = QDesktopServices::storageLocation(QDesktopServices::DataLocation)+"/exCopy/Scripts/";
	scriptpath = QDir::toNativeSeparators(scriptpath);
	QString scriptfile = QFileDialog::getOpenFileName(this,tr("   "), scriptpath, tr("  (*.js)"));
	
	//OPENFILENAME ofn;
	//TCHAR    file_name[MAX_PATH+1];

	//ZeroMemory(&ofn, sizeof(ofn));
	//ofn.lStructSize = sizeof(ofn);
	////ofn.hwndOwner = hwnd;
	//ofn.lpstrFile = file_name;
	//ofn.lpstrFile[0] = '\0';
	//ofn.nMaxFile = sizeof(file_name);
	//ofn.lpstrFilter = L"\0*.*\0";
	//ofn.nFilterIndex = 1;
	//ofn.lpstrFileTitle = NULL;
	//ofn.nMaxFileTitle = 0;
	//ofn.lpstrInitialDir = scriptpath.utf16();
	//ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

	//if (GetOpenFileName(&ofn))
 //      scriptfile = QString::fromStdWString(ofn.lpstrFile);

	if (!scriptfile.isEmpty())
	{
		QFile file(scriptfile);
		if (!file.open(QFile::ReadOnly))
		{
			QString msg = file.errorString();
			QMessageBox::warning(this, tr(""), msg);
			return;
		}
		QString content = file.readAll();
		file.close();
		scriptEdior->clear();
		scriptEdior->setText(content.toAscii());
		groupBox_6->setTitle(" "+scriptfile);
	}
}


void add_my_task::debug_script()
{
	QScriptEngineDebugger debugger;
	QScriptContext *context;
	debugger.standardWindow()->resize(800,600);
	//debugger.standardWindow()->move(1,1);
	debugger.attachTo(engine);
	debugger.action(QScriptEngineDebugger::InterruptAction)->trigger();
	
	QScriptValue result = engine->evaluate(scriptEdior->toPlainText());
	if (result.isError())
	{	
		QStringList backtrace = engine->uncaughtExceptionBacktrace();
		QMessageBox::information(0, " !", "  :\n"+result.toString()+" "+backtrace.join("\n"));	
	}
}

void add_my_task::save_script()
{
	QMenu menu;
	menu.addAction(tr("   "),this , SLOT(rewrite_exists_script_file()));
	menu.addAction(tr("   "),this , SLOT(create_new_script_file()));	
	menu.exec(QCursor::pos());	
}


void add_my_task::rewrite_exists_script_file()
{
	QString scriptpath = QDesktopServices::storageLocation(QDesktopServices::DataLocation)+"/exCopy/Scripts/";
	scriptpath = QDir::toNativeSeparators(scriptpath);
	QString scriptfile = QFileDialog::getOpenFileName(this,tr("   "), scriptpath, tr("  (*.js)"));
	if (!scriptfile.isEmpty())
	{
		QFile file(scriptfile);
		if (!file.open(QFile::WriteOnly | QFile::Text))
		{
			QString msg = file.errorString();
			QMessageBox::warning(this, tr("  "), msg);
			return;
		}
		QString content = scriptEdior->toPlainText();
		QByteArray block;
		block.append(content);
		file.write(block);
		file.close();
		QMessageBox::information(this, tr(" "), "  !");
	}
}

void add_my_task::create_new_script_file()
{
	
	bool ok;
	QString OldPath = QDir::currentPath();	
	QString scriptpath = QDesktopServices::storageLocation(QDesktopServices::DataLocation)+"/exCopy/Scripts/";
	scriptpath = QDir::toNativeSeparators(scriptpath);
	QDir::setCurrent(scriptpath);
	QString scriptfile = QInputDialog::getText(this, tr("  ,  ."), "  : "+scriptpath, QLineEdit::Normal, "", &ok);
	if (ok)	
	{
		scriptfile = scriptfile+".js";
		QFile file(scriptpath+scriptfile);

		if (file.exists())
		{
			QMessageBox::warning(this, tr(" !"), "   !");
			return;
		}

		if (!file.open(QFile::WriteOnly | QFile::Text))
		{
			QString msg = file.errorString();
			QMessageBox::warning(this, tr("  "), msg);
			return;
		}
		QString content = scriptEdior->toPlainText();
		QByteArray block;
		block.append(content);
		file.write(block);
		file.close();
		QMessageBox::information(this, tr(" "), "  !");
	}
	QDir::setCurrent(OldPath);
}

void add_my_task::on_add_Button_clicked()
{
	QMenu menu;
	menu.addAction(tr(" "),this , SLOT(add_files()));
	menu.addAction(tr(" "),this , SLOT(add_dirs()));
	menu.exec(QCursor::pos());	
}

void add_my_task::on_delete_Button_clicked()
{
	QList<QListWidgetItem *> selected;
	selected = list_files_and_dirs->selectedItems();

	foreach(QListWidgetItem *item, selected)	
		delete item;	
}

void add_my_task::on_dest_Button_clicked()
{
	QString directory = QFileDialog::getExistingDirectory(this,tr("  ?"),"",QFileDialog::ShowDirsOnly);
	if (!directory.isEmpty())
		dest_Edit->setText(directory);		
}

void add_my_task::select_before()
{
	QString runbeforefile = QFileDialog::getOpenFileName(this,tr(" "));
	if (!runbeforefile.isEmpty())
		execute_before->setText(runbeforefile);
}

void add_my_task::select_after()
{
	QString runafterfile = QFileDialog::getOpenFileName(this,tr(" "));
	if (!runafterfile.isEmpty())
		execute_after->setText(runafterfile);
}

void add_my_task::on_execute_before_customContextMenuRequested(QPoint)
{
	QMenu menu;
	menu.addAction(tr(" "),this , SLOT(select_before()));
	menu.exec(QCursor::pos());	
}

void add_my_task::on_execute_after_customContextMenuRequested(QPoint)
{
	QMenu menu;
	menu.addAction(tr(" "),this , SLOT(select_after()));
	menu.exec(QCursor::pos());	
}

void add_my_task::on_save_Button_clicked()
{
	emit query_disable_write();
	if (disable_write)
		return;

QStringList sourcefilesnames;
QString taskname = task_name->text();
QStandardItemModel ext(0,2), files(0,2), dirs(0,2);

for (int i=0;i<list_files_and_dirs->count();i++)
sourcefilesnames<<list_files_and_dirs->item(i)->text();

QString destdir = dest_Edit->text();

for (int i=0;i<listWidget->count();i++) //   
{
   if (listWidget->item(i)->data(Qt::UserRole).toString()=="ext")   
   {
		ext.insertRow(0);
		ext.setData(ext.index(0,0),listWidget->item(i)->text());
		ext.setData(ext.index(0,1),listWidget->item(i)->data(Qt::UserRole+1).toInt());
   }

	else
		if (listWidget->item(i)->data(Qt::UserRole).toString()=="file")
		{
			files.insertRow(0);
			files.setData(files.index(0,0),listWidget->item(i)->text());
			files.setData(files.index(0,1),listWidget->item(i)->data(Qt::UserRole+1).toInt());
		}
		else
			if (listWidget->item(i)->data(Qt::UserRole).toString()=="dir")
			{
				dirs.insertRow(0);
				dirs.setData(dirs.index(0,0),listWidget->item(i)->text());
				dirs.setData(dirs.index(0,1),listWidget->item(i)->data(Qt::UserRole+1).toInt());
			}
}

if (taskname.isEmpty() && sourcefilesnames.count() > 0)
{
		taskname = sourcefilesnames.at(0);
		task_name->setText(taskname);
}

bool ok = !taskname.isEmpty() && !destdir.isEmpty() && sourcefilesnames.count()!=0;

	if (ok)
	{		
		if (edit)
		{
			emit edit_and_update(itemindex, taskname, sourcefilesnames, destdir, create_add_dir->isChecked(), additional_dir->text(), clearDest->isChecked(), 
				notRewrite->isChecked(), reWrite->isChecked(), exDate->isChecked(), ex_Date_Copy->text(), conditionCombo->currentIndex(), ext, files, dirs, 
				exSize->isChecked(), conditionCombo_2->currentIndex(), ex_Size_Copy->text(), sizeCombo->currentIndex(), do_before->isChecked(), 
				execute_before->text(), execute_before_cmd->text(), execute_before_dir->text(), do_after->isChecked(), execute_after->text(), execute_after_cmd->text(),
				execute_after_dir->text(), cmp_md5->isChecked(), noRecursive->isChecked(), stackedWidget->currentIndex());
			    QString warn = "";
			    if (clearDest->isChecked())
				warn = "<p><b> !   '     '.       !</p></b>";
			    QMessageBox::information(this, tr(" "), " !"+warn);
		}
		else
		{
			emit add_and_update(taskname, sourcefilesnames, destdir, create_add_dir->isChecked(), additional_dir->text(), clearDest->isChecked(), 
				notRewrite->isChecked(), reWrite->isChecked(), exDate->isChecked(), ex_Date_Copy->text(), conditionCombo->currentIndex(), ext, files, dirs, 
				exSize->isChecked(), conditionCombo_2->currentIndex(), ex_Size_Copy->text(),sizeCombo->currentIndex(), do_before->isChecked(), 
				execute_before->text(), execute_before_cmd->text(), execute_before_dir->text(), do_after->isChecked(), execute_after->text(), 
				execute_after_cmd->text(), execute_after_dir->text(), cmp_md5->isChecked(), noRecursive->isChecked(), stackedWidget->currentIndex());
				QString warn = "";
				if (clearDest->isChecked())
				warn = "<p><b> !   '     '.       !</p></b>";
			    QMessageBox::information(this, tr(" "), "  !     . "+warn);
		}
	}
	else
		 QMessageBox::information(this, "!","     -  ,  ,  .");
}

void add_my_task::on_save_Button_2_clicked()
{
	emit query_disable_write();
	if (disable_write)
		return;
	
	if (task_name_2->text().isEmpty() && !scriptEdior->toPlainText().isEmpty())
		task_name_2->setText(" ");

	bool ok = ! task_name_2->text().isEmpty() && !scriptEdior->toPlainText().isEmpty();

	if (ok)
	{
		if (edit)
		{
			emit edit_script(itemindex, task_name_2->text(),scriptEdior->toPlainText(), stackedWidget->currentIndex());
			QMessageBox::information(this, tr(" "), "  !");
		}
		else
		{
			emit add_script(task_name_2->text(),scriptEdior->toPlainText(),stackedWidget->currentIndex());
			QMessageBox::information(this, tr(" "), "   !     .");
		}

		
	}
	else
		QMessageBox::information(this, "!","    -  , .");
}


void add_my_task::on_save_Button_3_clicked()
{	
	emit query_disable_write();
	if (disable_write)
		return;
	
	emit setup_scheduler(plan_items->currentItem()->data(Qt::UserRole).toLongLong(), enable_scheduler->isChecked(), comboBox->currentIndex(), Monday->isChecked(), Tuesday->isChecked(),
		Wednesday->isChecked(), Thursday->isChecked(), Friday->isChecked(), Saturday->isChecked(), Sunday->isChecked(), month_1->isChecked(), month_2->isChecked(), month_3->isChecked(),
		month_4->isChecked(),month_5->isChecked(), month_6->isChecked(), month_7->isChecked(), month_8->isChecked(), month_9->isChecked(), month_10->isChecked(), month_11->isChecked(),
		month_12->isChecked(),repeat_min->value(), dateEdit->date().toString("dd.MM.yyyy"),timeEdit->text());
	    QMessageBox::information(this, tr(" "), "   !");

	emit update_fase();
}

void add_my_task::on_save_Button_4_clicked()
{
	emit save_setup(autostart_planner->isChecked(), enable_concurrent_execution->isChecked(), ext_log->isChecked(), viewMessages->isChecked(), comboBox_2->currentIndex(), 
		Monday_1->isChecked(), Tuesday_1->isChecked(), Wednesday_1->isChecked(), Thursday_1->isChecked(), Friday_1->isChecked(), Saturday_1->isChecked(), Sunday_1->isChecked(), month_1_1->isChecked(), 
		month_1_2->isChecked(), month_1_3->isChecked(), month_1_4->isChecked(),month_1_5->isChecked(), month_1_6->isChecked(), month_1_7->isChecked(), month_1_8->isChecked(), month_1_9->isChecked(), 
		month_1_10->isChecked(), month_1_11->isChecked(), month_1_12->isChecked(),repeat_min_2->value(), dateEdit_2->date().toString("dd.MM.yyyy"),timeEdit_2->text(), clear_log->isChecked(), 
		stream_waiting->isChecked(), copy_empty_dir->isChecked(), do_limited_threds->isChecked(), count_limited_threads->value(), do_add_dir->isChecked(), doHide->isChecked());

	QMessageBox::information(this, tr(" "), " .""<p><b>      !</b></p>");			
}

void add_my_task::on_add_ext_clicked()
{
	bool ok;
	QString my_ext = QInputDialog::getText(this, tr(" "),tr("  :"), QLineEdit::Normal, "*.bak", &ok);
	if (ok)
	{
		QListWidgetItem *newItem = new QListWidgetItem;
		newItem->setData(Qt::UserRole,"ext");
		newItem->setText(my_ext);
		//newItem->setIcon(QIcon(":/icons/ext.png"));
		listWidget->insertItem(0, newItem);
		listWidget->setCurrentItem(newItem);
		listWidget->clearSelection();
		listWidget->currentItem()->setSelected(true);
	}	
}

void add_my_task::on_add_file_clicked()
{
	QStringList files=QFileDialog::getOpenFileNames(this,tr(" "),lastOpenDir,"All (*.*)",0,QFileDialog::ShowDirsOnly | QFileDialog::DontResolveSymlinks);
	if (!files.isEmpty())
	{
		foreach (QString item, files)
		{
			QListWidgetItem *newItem = new QListWidgetItem;
			newItem->setData(Qt::UserRole,"file");
			newItem->setText(item);
			//newItem->setIcon(QIcon(":/icons/files.png"));
			listWidget->insertItem(0, newItem);
			listWidget->setCurrentItem(newItem);
			listWidget->clearSelection();
			listWidget->currentItem()->setSelected(true);
		}
		lastOpenDir = QFileInfo(files.at(0)).absoluteDir().path();
	}
}

void add_my_task::on_add_dir_clicked()
{
	QString directory = QFileDialog::getExistingDirectory(this,tr(" "),lastOpenDir,QFileDialog::ShowDirsOnly);
	if (!directory.isEmpty())
	{
		QListWidgetItem *newItem = new QListWidgetItem;
		newItem->setData(Qt::UserRole,"dir");
		newItem->setText(directory);
		//newItem->setIcon(QIcon(":/icons/folder.png"));
		listWidget->insertItem(0, newItem);
		listWidget->setCurrentItem(newItem);
		listWidget->clearSelection();
		listWidget->currentItem()->setSelected(true);

		QDir updir(directory);
		updir.cdUp();
		lastOpenDir = updir.path();	
	}
}

void add_my_task::on_delete_from_list_clicked()
{
	QList<QListWidgetItem *> selected;
	selected = listWidget->selectedItems();

	foreach(QListWidgetItem *item, selected)		
		delete item;

	if (listWidget->count()>0)
	{
		listWidget->setCurrentItem(0);
		listWidget->item(0)->setSelected(true);

		if (listWidget->item(0)->data(Qt::UserRole+1).toInt() == 2)
			include_filter->setChecked(1);
		else
			if (listWidget->item(0)->data(Qt::UserRole+1).toInt() == 1)
				exclude_filter->setChecked(1);
			else
				if (listWidget->item(0)->data(Qt::UserRole+1).toInt() == 0)
				{
					include_filter->setChecked(0);
					exclude_filter->setChecked(0);
				}
	}
	else
	{
		include_filter->setChecked(0);
		exclude_filter->setChecked(0);
	}
}


void add_my_task::on_reWrite_stateChanged(int state)
{
	if (state == 2 && notRewrite->isChecked())
		notRewrite->setCheckState(Qt::Unchecked);
}


void add_my_task::on_notRewrite_stateChanged(int state)
{
	if (state == 2 && reWrite->isChecked())
		reWrite->setCheckState(Qt::Unchecked);

}

void add_my_task::on_include_filter_stateChanged(int state)
{
	if (state == 2 && exclude_filter->isChecked())
		exclude_filter->setCheckState(Qt::Unchecked);
}

void add_my_task::on_exclude_filter_stateChanged(int state)
{
	if (state == 2 && include_filter->isChecked())
		include_filter->setCheckState(Qt::Unchecked);

}

void add_my_task::set_item_filter_state()
{
	QList<QListWidgetItem *> selected;
	selected = listWidget->selectedItems();

	if (selected.count() == 0)
	{
		QMessageBox::information(this, "!","   , , .");
		return;
	}

	foreach(QListWidgetItem *item, selected)
	{
		if (include_filter->isChecked())
			item->setData(Qt::UserRole+1, 2); //  
		  else
			 if (exclude_filter->isChecked())
				item->setData(Qt::UserRole+1, 1); //  
			  else
				  item->setData(Qt::UserRole+1, 0); //  
	}
}

void add_my_task::set_CheckBox_state(int currentindex)
{
	if (currentindex == -1)
		return;

	if (listWidget->item(currentindex)->data(Qt::UserRole+1).toInt() == 2)
	include_filter->setChecked(1);
	else
		if (listWidget->item(currentindex)->data(Qt::UserRole+1).toInt() == 1)
			exclude_filter->setChecked(1);
		 else
			  if (listWidget->item(currentindex)->data(Qt::UserRole+1).toInt() == 0)
				{
				include_filter->setChecked(0);
				exclude_filter->setChecked(0);
				}
}

void add_my_task::get_top_item_state()
{
	set_CheckBox_state(0);
}

void add_my_task::on_listWidget_itemDoubleClicked(QListWidgetItem * item)
{
	bool ok, file_or_dir;
	QString caption;
	file_or_dir = false;

	if (item->data(Qt::UserRole).toString() == "ext")
	{
		caption="   :";
	}
	else
		if (item->data(Qt::UserRole).toString() == "file")
		{
			caption="   :";
			file_or_dir = true;
		}
		else
			if (item->data(Qt::UserRole).toString() == "dir")
			{
				caption=" :";
				file_or_dir = true;
			}
	
	QString new_text = QInputDialog::getText(this, tr(""), caption, QLineEdit::Normal, item->text(), &ok);
	if (ok)	
	{
		if (file_or_dir)
		{
			int pos = 0;
			QRegExp rx("[^*?<>|]+");
			QRegExpValidator v(rx, 0);
			if (v.validate(new_text, pos) != QValidator::Acceptable)
				QMessageBox::information(this, "","        .");			
				item->setText(new_text);	
		}
		else
			item->setText(new_text);
	}
}

void add_my_task::on_comboBox_currentIndexChanged(int index)
{
	switch	(index)
	{	
	case 0:
		{
			stackedWidget_2->setCurrentIndex(0);
			stackedWidget_2->setEnabled(false);
			Monday->setChecked(false);
			Tuesday->setChecked(false);
			Wednesday->setChecked(false);
			Thursday->setChecked(false);
			Friday->setChecked(false);
			Saturday->setChecked(false);
			Sunday->setChecked(false);
			dateEdit->setDisplayFormat("dd.MM.yyyy");
			dateEdit->setEnabled(true);
			timeEdit->setEnabled(true);
		}
		break;
	case 1:
		{
			stackedWidget_2->setCurrentIndex(0);
			stackedWidget_2->setEnabled(true);
			Monday->setChecked(true);
			Tuesday->setChecked(true);
			Wednesday->setChecked(true);
			Thursday->setChecked(true);
			Friday->setChecked(true);
			Saturday->setChecked(true);
			Sunday->setChecked(true);
			dateEdit->setDisplayFormat("dd.MM.yyyy");
			dateEdit->setEnabled(false);
			timeEdit->setEnabled(true);
		}
		break;
	case 2:
		{
			stackedWidget_2->setCurrentIndex(1);
			stackedWidget_2->setEnabled(true);
			month_1->setChecked(true);
			month_2->setChecked(true);
			month_3->setChecked(true);
			month_4->setChecked(true);
			month_5->setChecked(true);
			month_6->setChecked(true);
			month_7->setChecked(true);
			month_8->setChecked(true);
			month_9->setChecked(true);
			month_10->setChecked(true);
			month_11->setChecked(true);
			month_12->setChecked(true);
			dateEdit->setDisplayFormat(" dd ");			 
			dateEdit->setEnabled(true);
			timeEdit->setEnabled(true);
		}
		break;
	case 3:
		{
			stackedWidget_2->setCurrentIndex(0);
			stackedWidget_2->setEnabled(false);
			Monday->setChecked(false);
			Tuesday->setChecked(false);
			Wednesday->setChecked(false);
			Thursday->setChecked(false);
			Friday->setChecked(false);
			Saturday->setChecked(false);
			Sunday->setChecked(false);		
			dateEdit->setDisplayFormat("dd.MM.yyyy");
			dateEdit->setEnabled(true);
			timeEdit->setEnabled(true);
		}
		break;
	case 4:
		{
			stackedWidget_2->setCurrentIndex(2);
			stackedWidget_2->setEnabled(true);
			Monday->setChecked(false);
			Tuesday->setChecked(false);
			Wednesday->setChecked(false);
			Thursday->setChecked(false);
			Friday->setChecked(false);
			Saturday->setChecked(false);
			Sunday->setChecked(false);
			dateEdit->setDisplayFormat("dd.MM.yyyy");
			dateEdit->setEnabled(false);
			timeEdit->setEnabled(false);
		}
		break;
	}
}

void  add_my_task::on_comboBox_2_currentIndexChanged(int index)
{
	switch	(index)
	{	
	case 0:
		{
			stackedWidget_3->setCurrentIndex(0);
			stackedWidget_3->setEnabled(false);
			Monday_1->setChecked(false);
			Tuesday_1->setChecked(false);
			Wednesday_1->setChecked(false);
			Thursday_1->setChecked(false);
			Friday_1->setChecked(false);
			Saturday_1->setChecked(false);
			Sunday_1->setChecked(false);
			dateEdit_2->setDisplayFormat("dd.MM.yyyy");
			dateEdit_2->setEnabled(true);
			timeEdit_2->setEnabled(true);
		}
		break;
	case 1:
		{
			stackedWidget_3->setCurrentIndex(0);
			stackedWidget_3->setEnabled(true);
			Monday_1->setChecked(true);
			Tuesday_1->setChecked(true);
			Wednesday_1->setChecked(true);
			Thursday_1->setChecked(true);
			Friday_1->setChecked(true);
			Saturday_1->setChecked(true);
			Sunday_1->setChecked(true);
			dateEdit_2->setDisplayFormat("dd.MM.yyyy");
			dateEdit_2->setEnabled(false);
			timeEdit_2->setEnabled(true);
		}
		break;
	case 2:
		{
			stackedWidget_3->setCurrentIndex(1);
			stackedWidget_3->setEnabled(true);
			month_1_1->setChecked(true);
			month_1_2->setChecked(true);
			month_1_3->setChecked(true);
			month_1_4->setChecked(true);
			month_1_5->setChecked(true);
			month_1_6->setChecked(true);
			month_1_7->setChecked(true);
			month_1_8->setChecked(true);
			month_1_9->setChecked(true);
			month_1_10->setChecked(true);
			month_1_11->setChecked(true);
			month_1_12->setChecked(true);
			dateEdit_2->setDisplayFormat(" dd ");			 
			dateEdit_2->setEnabled(true);
			timeEdit_2->setEnabled(true);
		}
		break;
	case 3:
		{
			stackedWidget_3->setCurrentIndex(0);
			stackedWidget_3->setEnabled(false);
			Monday_1->setChecked(false);
			Tuesday_1->setChecked(false);
			Wednesday_1->setChecked(false);
			Thursday_1->setChecked(false);
			Friday_1->setChecked(false);
			Saturday_1->setChecked(false);
			Sunday_1->setChecked(false);		
			dateEdit_2->setDisplayFormat("dd.MM.yyyy");
			dateEdit_2->setEnabled(true);
			timeEdit_2->setEnabled(true);
		}
		break;
	case 4:
		{
			stackedWidget_3->setCurrentIndex(2);
			stackedWidget_3->setEnabled(true);
			Monday_1->setChecked(false);
			Tuesday_1->setChecked(false);
			Wednesday_1->setChecked(false);
			Thursday_1->setChecked(false);
			Friday_1->setChecked(false);
			Saturday_1->setChecked(false);
			Sunday_1->setChecked(false);
			dateEdit_2->setDisplayFormat("dd.MM.yyyy");
			dateEdit_2->setEnabled(false);
			timeEdit_2->setEnabled(false);
		}
		break;
	}
}

void add_my_task::on_conditionCombo_currentIndexChanged(int index)
{
	if (index == 3)
		ex_Date_Copy->setDisplayFormat("yy");
	else
		ex_Date_Copy->setDisplayFormat("dd.MM.yyyy");
}

void add_my_task::on_list_files_and_dirs_itemDoubleClicked(QListWidgetItem * item)
{
	bool ok;
	QString new_item = QInputDialog::getText(this, tr(""), "  :", QLineEdit::Normal, item->text(), &ok);
	if (ok)					
		item->setText(new_item);	
}

void add_my_task::on_enable_scheduler_stateChanged(int state)
{
	if (state)
	{
		label_3->setEnabled(true);
		comboBox->setEnabled(true);
		groupBox_9->setEnabled(true);
		stackedWidget_2->setEnabled(true);

		int old_index = comboBox->currentIndex();
		comboBox->setCurrentIndex(-1); //  on_comboBox_currentIndexChanged     
		comboBox->setCurrentIndex(old_index);
	}
	else
	{
		label_3->setEnabled(false);
		comboBox->setEnabled(false);
		groupBox_9->setEnabled(false);		
		stackedWidget_2->setEnabled(false);
	}	
}

void add_my_task::update_interfase_scheduler(const qint64 &index)
{
	QSqlQuery myquery;
	myquery.prepare("SELECT * FROM GET_SCHEDULER_DATA(?)");
	myquery.bindValue(0, index);

	if (!myquery.exec())
	{
		QMessageBox::information(this, "","   (  GET_SCHEDULER_DATA)");
		return;
	}
	else
	{
		myquery.first();

		enable_scheduler->setChecked(myquery.value(0).toInt());

		if (!enable_scheduler->isChecked())
		{
			label_3->setEnabled(false);
			comboBox->setEnabled(false);
			groupBox_9->setEnabled(false);
			stackedWidget_2->setEnabled(false);
			return;
		}
		else
		{
			label_3->setEnabled(true);
			comboBox->setEnabled(true);
			groupBox_9->setEnabled(true);			
			stackedWidget_2->setEnabled(true);

			comboBox->setCurrentIndex(-1); //  on_comboBox_currentIndexChanged
			comboBox->setCurrentIndex(myquery.value(1).toInt());
		}

		Monday->setChecked(myquery.value(2).toInt());
		Tuesday->setChecked(myquery.value(3).toInt());
		Wednesday->setChecked(myquery.value(4).toInt());
		Thursday->setChecked(myquery.value(5).toInt());
		Friday->setChecked(myquery.value(6).toInt());
		Saturday->setChecked(myquery.value(7).toInt());
		Sunday->setChecked(myquery.value(8).toInt());

		month_1->setChecked(myquery.value(9).toInt());
		month_2->setChecked(myquery.value(10).toInt());
		month_3->setChecked(myquery.value(11).toInt());
		month_4->setChecked(myquery.value(12).toInt());
		month_5->setChecked(myquery.value(13).toInt());
		month_6->setChecked(myquery.value(14).toInt());
		month_7->setChecked(myquery.value(15).toInt());
		month_8->setChecked(myquery.value(16).toInt());
		month_9->setChecked(myquery.value(17).toInt());
		month_10->setChecked(myquery.value(18).toInt());
		month_11->setChecked(myquery.value(19).toInt());
		month_12->setChecked(myquery.value(20).toInt());

		repeat_min->setValue(myquery.value(21).toInt());

		dateEdit->setDate(QDate::fromString(myquery.value(22).toString(),"dd.MM.yyyy"));
		timeEdit->setTime(QTime::fromString(myquery.value(23).toString(),"HH:mm"));
	}
}

void add_my_task::on_plan_items_currentItemChanged(QListWidgetItem * current, QListWidgetItem * previous)
{
	if (current)
	{
		if (previous)
			emit setup_scheduler(previous->data(Qt::UserRole).toLongLong(), enable_scheduler->isChecked(), comboBox->currentIndex(), Monday->isChecked(), Tuesday->isChecked(),
			Wednesday->isChecked(), Thursday->isChecked(), Friday->isChecked(), Saturday->isChecked(), Sunday->isChecked(), month_1->isChecked(), month_2->isChecked(), month_3->isChecked(),
			month_4->isChecked(),month_5->isChecked(), month_6->isChecked(), month_7->isChecked(), month_8->isChecked(), month_9->isChecked(), month_10->isChecked(), month_11->isChecked(),
			month_12->isChecked(),repeat_min->value(), dateEdit->date().toString("dd.MM.yyyy"),timeEdit->text());
	        update_interfase_scheduler(current->data(Qt::UserRole).toLongLong());	
	}
}

void add_my_task::on_plan_items_customContextMenuRequested(QPoint)
{
	QMenu menu;
	menu.addAction(tr("      "),this , SLOT(replace_all_param()));
	menu.exec(QCursor::pos());	
}

void add_my_task::replace_all_param()
{
	emit replace_all_param(enable_scheduler->isChecked(), comboBox->currentIndex(), Monday->isChecked(), Tuesday->isChecked(),
		Wednesday->isChecked(), Thursday->isChecked(), Friday->isChecked(), Saturday->isChecked(), Sunday->isChecked(), month_1->isChecked(), month_2->isChecked(), month_3->isChecked(),
		month_4->isChecked(),month_5->isChecked(), month_6->isChecked(), month_7->isChecked(), month_8->isChecked(), month_9->isChecked(), month_10->isChecked(), month_11->isChecked(),
		month_12->isChecked(),repeat_min->value(), dateEdit->date().toString("dd.MM.yyyy"),timeEdit->text());
	    QMessageBox::information(this, tr(" "), "  !");
}

void add_my_task::set_current_plan_items(const qint64 & index)
{
	for (int i=0;i<plan_items->count();i++)
	{
		if (plan_items->item(i)->data(Qt::UserRole).toLongLong() == index)
		{
			plan_items->setCurrentRow(i);
			break;
		}
	}
}

void add_my_task::closeEvent( QCloseEvent * event )
{
	if (stackedWidget->currentIndex() == 3)
	{
		emit query_disable_write();
		if (!disable_write)			
		   emit update_fase();
	}
	edit = false;
	event->accept();	
}