#include "scripting.h"


ThreadScript::ThreadScript(QObject *parent):QThread(parent)
{
	my_ScriptWrapper = new ScriptWrapper;
	engine = new QScriptEngine;
	//engine.setProcessEventsInterval(20);
	my_log.clear();
	
	QScriptValue ret = engine->importExtension("qt.core"); 
	if (ret.isError())
	{
		emit setTaskStatus(IndexThread, "", true);
		my_log.append("   qtscript_core.dll");

	}
	
	QScriptValue my_Wrapper = engine->newQObject(my_ScriptWrapper);
	engine->globalObject().setProperty("engine", my_Wrapper);
	if (my_Wrapper.isError())
	{
		emit setTaskStatus(IndexThread, "", true);
		my_log.append("  ");
	}

	connect(this, SIGNAL(finished ()), this, SLOT(do_stopped_()));
}

void ThreadScript::do_stopped_()
{
	emit thread_stopped(IndexThread);
}

void ThreadScript::setData(const QString &script_text_, const qint64 &IndexThread_)
{
	
	script_text = script_text_;
	IndexThread = IndexThread_;
}

void ThreadScript::run()
{
    my_ScriptWrapper->setstart();	

	QTime t;
		t.start();

	emit setTaskStatus(IndexThread, "", false);
	my_log.append("    "+QDateTime::currentDateTime().toString("dd.MM.yyyy hh:mm:ss"));

	QScriptValue result = engine->evaluate(script_text);
	if (result.isError())
	{	
		QStringList backtrace = engine->uncaughtExceptionBacktrace();
		emit setTaskStatus(IndexThread, "", true);
		my_log.append("  : "+result.toString()+" "+backtrace.join("\n"));
	}

	int hours,minutes,seconds;
	int msecunds = t.elapsed();	

	seconds = (msecunds / 1000); 
	hours = seconds / 3600;
	minutes  = (seconds % 3600) / 60;
	seconds %= 60;
	
	my_log.append("  : "+QString::number(hours)+" ., "+QString::number(minutes)+" ., "+QString::number(seconds)+" ." + " : " + QDateTime::currentDateTime().toString("dd.MM.yyyy hh:mm:ss"));
	emit save_thread_log(IndexThread, my_log);
}

void ThreadScript::dostart()
{
	if (!isRunning()) 
		start();
}

void ThreadScript::stop()
{
	my_ScriptWrapper->quit();
	engine->abortEvaluation();		
	my_log.append("  ");
}

ThreadScript::~ThreadScript()
{

	if (my_ScriptWrapper)
	{
		delete my_ScriptWrapper;
		my_ScriptWrapper = 0;
	}

	if (engine)
	{
		delete engine;
		engine = 0;
	}

}

qint64 ThreadScript::getindex()
{
	return IndexThread;
}

QStringList ThreadScript::getlog()
{
	return my_log;
}

